program clock;

uses
  Windows, Sysutils;

{$apptype console }

function ExecuteAndWait(const cmdline : string) : boolean;
var
  si : TStartupInfo;
  pi : TProcessInformation;
begin
  FillChar(si, sizeof(si), 0);
  si.cb := sizeof(si);

  // Try to start the process
  Result:=CreateProcess(nil, PChar(cmdline), nil, nil, False, 0, nil, nil, si, pi);

  // If success wait for it to exit
  if Result
  then begin
    Result:= WaitForSingleObject(pi.hProcess, INFINITE) = WAIT_OBJECT_0;
    CloseHandle(pi.hProcess);
    CloseHandle(pi.hThread);
  end;
end;

{
CloseHandle() must be called on the open process and thread handles obtained
from the CreateProcess() call.  They are not returned as a function return,
but rather through the PROCESS_INFORMATION structure passed in the call so
it appears many people seem to miss this issue.  These handles are reference
counted and the kernel level objects that manage these will not be freed
until the count reaches 0.  At worst, this is an application lifetime leak,
but it's still quite significant and heaven help you if you're writing a
24/7 application, server or service!


See:
http://msdn.microsoft.com/library/partbook/win98dh/makingmodifyingprocesses.htm

QUOTED FROM ABOVE URL:
"If your program does not make use of the handles for the new process and its
primary thread, you should close both right away. Otherwise, even if the
PROCESS_INFORMATION structure is a local variable and goes out of scope,
abandoning whatever it contained, the two object entries remain in your
processs object table and the system counts them as open handles.

Because the size of physical system memory limits the total number of
processes that can be created, be sure to check for error returns. As an
example, one of the early beta versions of Windows NT allowed a 16MB machine
to create about 40 processes before failing for lack of memory. This limit
will vary from version to version and machine to machine, but it is finite.
Memory shortage can also cause CreateThread to fail, though threads consume
significantly fewer resources than processes."

Jim Burns - http://www.technologydynamics.com
}


var t0,t1,dt:TDateTime; s:string; i:integer;
begin
  Writeln('Clock run times (c) 2004 jan 01 by Maarten Pennings');
  if (ParamCount<1) or (ParamStr(1)='/?') or (ParamStr(1)='/h') or (ParamStr(1)='/help')
  then begin
    Writeln('CLOCK [<path>]<prog> [<args>]');
    Exit
  end;

  s:=ParamStr(1);
  for i:=2 to ParamCount do s:=s+' '+ParamStr(i);
  Writeln('Starting "', s ,'"' );

  t0:=Now;
  if not ExecuteAndWait(s)
  then begin
    Writeln('Start failed');
    Exit
  end;
  t1:=Now;
  writeln('Ended');
  dt:=(t1-t0)*24*60*60*1000;
  if dt<10000 then Writeln(Format('%.0f msecs',[dt]))
  else Writeln(Format('%.3f secs',[dt/1000]))
end.
